@@ -166,7 +166,8 @@ class GXDLMS:
             raise ValueError("Invalid Server Address.")
 
     @classmethod
-    def appendData(cls, obj, index, bb, value):
+    #pylint: disable=too-many-arguments
+    def appendData(cls, settings, obj, index, bb, value):
         tp = obj.getDataType(index)
         if tp == DataType.ARRAY:
             if isinstance(value, bytes):
@@ -179,9 +180,9 @@ class GXDLMS:
             if isinstance(value, str) and tp == DataType.OCTET_STRING:
                 ui = obj.getUIDataType(index)
                 if ui == DataType.STRING:
-                    _GXCommon.setData(bb, tp, value.encode())
+                    _GXCommon.setData(settings, bb, tp, value.encode())
                     return
-        _GXCommon.setData(bb, tp, value)
+        _GXCommon.setData(settings, bb, tp, value)
 
     #
     # Get used glo message.
@@ -335,7 +336,7 @@ class GXDLMS:
                     reply.setUInt8(DataType.NONE)
                 else:
                     pos = len(reply)
-                    _GXCommon.setData(reply, DataType.OCTET_STRING, p.getTime())
+                    _GXCommon.setData(p.settings, reply, DataType.OCTET_STRING, p.getTime())
                     if p.command != Command.EVENT_NOTIFICATION:
                         reply.move(pos + 1, pos, len(reply) - pos - 1)
                 cls.multipleBlocks(p, reply, ciphering)
@@ -528,7 +529,7 @@ class GXDLMS:
                 assert not p.settings.maxPduSize < len(reply)
             while reply.position != len(reply):
                 if p.settings.interfaceType == InterfaceType.WRAPPER:
-                    messages.append(GXDLMS.getWrapperFrame(p.settings, reply))
+                    messages.append(GXDLMS.getWrapperFrame(p.settings, p.command, reply))
                 elif p.settings.interfaceType == InterfaceType.HDLC:
                     messages.append(GXDLMS.getHdlcFrame(p.settings, frame_, reply))
                     if reply.position != len(reply):
@@ -557,7 +558,7 @@ class GXDLMS:
                 assert not p.settings.maxPduSize < len(reply)
             while reply.position != len(reply):
                 if p.settings.interfaceType == InterfaceType.WRAPPER:
-                    messages.append(cls.getWrapperFrame(p.settings, reply))
+                    messages.append(cls.getWrapperFrame(p.settings, p.command, reply))
                 elif p.settings.interfaceType == InterfaceType.HDLC:
                     messages.append(cls.getHdlcFrame(p.settings, frame_, reply))
                     if reply.position != len(reply):
@@ -620,7 +621,7 @@ class GXDLMS:
                 reply.setUInt8(DataType.NONE)
             else:
                 pos = len(reply)
-                _GXCommon.setData(reply, DataType.OCTET_STRING, p.time)
+                _GXCommon.setData(p.settings, reply, DataType.OCTET_STRING, p.time)
                 reply.move(pos + 1, pos, len(reply) - pos - 1)
             _GXCommon.setObjectCount(p.count, reply)
             reply.set(p.attributeDescriptor)
@@ -700,12 +701,15 @@ class GXDLMS:
         return bb.array()
 
     @classmethod
-    def getWrapperFrame(cls, settings, data):
+    def getWrapperFrame(cls, settings, command, data):
         bb = GXByteBuffer()
         bb.setUInt16(1)
         if settings.isServer:
             bb.setUInt16(settings.serverAddress)
-            bb.setUInt16(settings.clientAddress)
+            if settings.PushClientAddress != 0 and command in (Command.DATA_NOTIFICATION, Command.EVENT_NOTIFICATION):
+                bb.setUInt16(settings.pushClientAddress)
+            else:
+                bb.setUInt16(settings.clientAddress)
         else:
             bb.setUInt16(settings.clientAddress)
             bb.setUInt16(settings.serverAddress)
@@ -731,7 +735,10 @@ class GXDLMS:
         primaryAddress = None
         secondaryAddress = None
         if settings.isServer:
-            primaryAddress = cls.getAddressBytes(settings.clientAddress, 1)
+            if frame_ == 0x13 and settings.pushClientAddress != 0:
+                primaryAddress = cls.getAddressBytes(settings.pushClientAddress, 1)
+            else:
+                primaryAddress = cls.getAddressBytes(settings.clientAddress, 1)
             secondaryAddress = cls.getAddressBytes(settings.serverAddress, settings.serverAddressSize)
         else:
             primaryAddress = cls.getAddressBytes(settings.serverAddress, settings.serverAddressSize)
@@ -846,8 +853,11 @@ class GXDLMS:
                 notify.serverAddress = addresses[0]
         # HDLC control fields
         cf = reply.getUInt8()
-        if notify and cf == 0x13:
+        if not isNotify and notify and cf == 0x13:
             isNotify = True
+            notify.clientAddress = addresses[1]
+            notify.serverAddress = addresses[0]
+
         if (frame_ & 0x8) != 0:
             if isNotify:
                 notify.moreData = notify.moreData | RequestTypes.FRAME
@@ -1101,13 +1111,20 @@ class GXDLMS:
         data = reply.data
         pos = 0
         cnt = reply.getTotalCount()
-        first = reply.getTotalCount() == 0 or reply.getCommandType() == SingleReadResponse.DATA_BLOCK_RESULT
+        first = cnt == 0 or reply.commandType == SingleReadResponse.DATA_BLOCK_RESULT
         if first:
             cnt = _GXCommon.getObjectCount(reply.data)
             reply.totalCount = cnt
         type_ = 0
         values = None
         if cnt != 1:
+            #Parse data after all data is received when readlist is used.
+            if reply.isMoreData():
+                cls.getDataFromBlock(reply.data, 0)
+                return False
+            if not first:
+                reply.data.position = 0
+                first = True
             values = list()
             if isinstance(reply.value, list):
                 values.append(reply.value)
@@ -1115,12 +1132,6 @@ class GXDLMS:
         if reply.xml:
             reply.xml.appendStartTag(Command.READ_RESPONSE, "Qty", reply.xml.integerToHex(cnt, 2))
         while pos != cnt:
-            if data.available() == 0:
-                if cnt != 1:
-                    cls.getDataFromBlock(reply.data, 0)
-                    reply.value = values
-                    reply.readPosition = data.position
-                return False
             if first:
                 type_ = data.getUInt8()
                 reply.commandType = type_
@@ -1135,7 +1146,7 @@ class GXDLMS:
                     reply.xml.appendStartTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
                     di = _GXDataInfo()
                     di.xml = (reply.xml)
-                    _GXCommon.getData(reply.data, di)
+                    _GXCommon.getData(settings, reply.data, di)
                     reply.xml.appendEndTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
                     if standardXml:
                         reply.xml.appendEndTag(TranslatorTags.CHOICE)
@@ -1144,16 +1155,6 @@ class GXDLMS:
                 else:
                     reply.readPosition = data.position
                     cls.getValueFromData(settings, reply)
-                    if data.position == reply.readPosition:
-                        index2 = index
-                        if cnt != 1 and reply.totalCount == 0:
-                            index2 += 1
-                        reply.totalCount = 0
-                        data.position = index2
-                        cls.getDataFromBlock(reply.data, 0)
-                        reply.value = None
-                        reply.commandType = SingleReadResponse.DATA_BLOCK_RESULT
-                        return False
                     data.position = reply.readPosition
                     values.append(reply.value)
                     reply.value = None
@@ -1193,7 +1194,7 @@ class GXDLMS:
         return TranslatorSimpleTags.errorCodeToString(value)
 
     @classmethod
-    def handleMethodResponse(cls, data):
+    def handleMethodResponse(cls, settings, data):
         type_ = int(data.data.getUInt8())
         invoke = data.data.getUInt8()
         if data.xml:
@@ -1234,7 +1235,7 @@ class GXDLMS:
                         data.xml.appendStartTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
                         di = _GXDataInfo()
                         di.xml = (data.xml)
-                        _GXCommon.getData(data.data, di)
+                        _GXCommon.getData(settings, data.data, di)
                         data.xml.appendEndTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
                     data.xml.appendEndTag(TranslatorTags.RETURN_PARAMETERS)
                     if standardXml:
@@ -1273,7 +1274,7 @@ class GXDLMS:
         cls.getDataFromBlock(reply.data, index)
 
     @classmethod
-    def handleAccessResponse(cls, reply):
+    def handleAccessResponse(cls, settings, reply):
         data = reply.data
         invokeId = reply.data.getUInt32()
         len_ = reply.data.getUInt8()
@@ -1281,7 +1282,7 @@ class GXDLMS:
         if len_ != 0:
             tmp = bytearray(len_)
             data.get(tmp)
-            reply.time = _GXCommon.changeType(tmp, DataType.DATETIME)
+            reply.time = _GXCommon.changeType(settings, tmp, DataType.DATETIME)
         if reply.xml:
             reply.xml.appendStartTag(Command.ACCESS_RESPONSE)
             reply.xml.appendLine(TranslatorTags.LONG_INVOKE_ID, None, reply.xml.integerToHex(invokeId, 8))
@@ -1298,7 +1299,7 @@ class GXDLMS:
                     reply.xml.appendStartTag(Command.WRITE_REQUEST, SingleReadResponse.DATA)
                 di = _GXDataInfo()
                 di.xml = (reply.xml)
-                _GXCommon.getData(reply.data, di)
+                _GXCommon.getData(settings, reply.data, di)
                 if reply.xml.outputType == TranslatorOutputType.STANDARD_XML:
                     reply.xml.appendEndTag(Command.WRITE_REQUEST, SingleReadResponse.DATA)
                 pos += 1
@@ -1342,7 +1343,7 @@ class GXDLMS:
                 dt = DataType.DATE
             info = _GXDataInfo()
             info.type_ = dt
-            reply.time = _GXCommon.getData(GXByteBuffer(tmp), info)
+            reply.time = _GXCommon.getData(settings, GXByteBuffer(tmp), info)
         if reply.xml:
             reply.xml.appendStartTag(Command.DATA_NOTIFICATION)
             reply.xml.appendLine(TranslatorTags.LONG_INVOKE_ID, None, reply.xml.integerToHex(invokeId, 8))
@@ -1353,7 +1354,7 @@ class GXDLMS:
             reply.xml.appendStartTag(TranslatorTags.DATA_VALUE)
             di = _GXDataInfo()
             di.xml = (reply.xml)
-            _GXCommon.getData(reply.data, di)
+            _GXCommon.getData(settings, reply.data, di)
             reply.xml.appendEndTag(TranslatorTags.DATA_VALUE)
             reply.xml.appendEndTag(TranslatorTags.NOTIFICATION_BODY)
             reply.xml.appendEndTag(Command.DATA_NOTIFICATION)
@@ -1442,7 +1443,7 @@ class GXDLMS:
                     di = _GXDataInfo()
                     di.xml = (reply.xml)
                     reply.xml.appendStartTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
-                    _GXCommon.getData(reply.data, di)
+                    _GXCommon.getData(settings, reply.data, di)
                     reply.xml.appendEndTag(Command.READ_RESPONSE, SingleReadResponse.DATA)
                 else:
                     reply.readPosition = reply.data.position
@@ -1476,7 +1477,7 @@ class GXDLMS:
                     reply.xml.appendStartTag(TranslatorTags.DATA)
                     di = _GXDataInfo()
                     di.xml = (reply.xml)
-                    _GXCommon.getData(reply.data, di)
+                    _GXCommon.getData(settings, reply.data, di)
                     reply.xml.appendEndTag(TranslatorTags.DATA)
             else:
                 cls.getDataFromBlock(reply.data, 0)
@@ -1548,8 +1549,21 @@ class GXDLMS:
         bc = data.data.getUInt8()
         data.streaming = (bc & 0x40) != 0
         windowSize = int(bc & 0x3F)
-        data.blockNumber = data.data.getUInt16()
-        data.blockNumberAck = data.data.getUInt16()
+        bn = data.data.getUInt16()
+        bna = data.data.getUInt16()
+
+        if not data.xml:
+            #Remove existing data when first block is received.
+            if bn == 1:
+                index = 0
+            elif bna != settings.blockIndex - 1:
+                #If this block is already received.
+                data.data.size = index
+                data.command = Command.NONE
+                return
+
+        data.blockNumber = bn
+        data.blockNumberAck = bna
         settings.blockNumberAck = data.blockNumber
         data.command = Command.NONE
         len_ = _GXCommon.getObjectCount(data.data)
@@ -1615,9 +1629,9 @@ class GXDLMS:
             elif cmd == Command.WRITE_RESPONSE:
                 cls.handleWriteResponse(data)
             elif cmd == Command.METHOD_RESPONSE:
-                cls.handleMethodResponse(data)
+                cls.handleMethodResponse(settings, data)
             elif cmd == Command.ACCESS_RESPONSE:
-                cls.handleAccessResponse(data)
+                cls.handleAccessResponse(settings, data)
             elif cmd == Command.GENERAL_BLOCK_TRANSFER:
                 if data.xml or (not settings.isServer and (data.moreData & RequestTypes.FRAME) == 0):
                     cls.handleGbt(settings, data)
@@ -1684,6 +1698,10 @@ class GXDLMS:
                     Command.GENERAL_GLO_CIPHERING, Command.GENERAL_DED_CIPHERING):
                     data.data.position = data.cipherIndex
                     cls.getPdu(settings, data)
+                if cmd == Command.READ_RESPONSE and data.totalCount > 1:
+                    if not cls.handleReadResponse(settings, data, 0):
+                        return
+
         if cmd == Command.READ_RESPONSE and data.commandType == SingleReadResponse.DATA_BLOCK_RESULT and \
             (data.moreData & RequestTypes.FRAME) != 0:
             return
@@ -1804,7 +1822,7 @@ class GXDLMS:
         index = data.position
         data.position = reply.readPosition
         try:
-            value = _GXCommon.getData(data, info)
+            value = _GXCommon.getData(settings, data, info)
             if value is not None:
                 if not isinstance(value, list):
                     reply.valueType = info.type_
