@@ -138,9 +138,9 @@ class GXDLMSModemConfiguration(GXDLMSObject, IGXDLMSBase):
                     data.setUInt8(DataType.STRUCTURE)
                     data.setUInt8(3)
                     #  Count
-                    _GXCommon.setData(data, DataType.OCTET_STRING, _GXCommon.getBytes(it.request))
-                    _GXCommon.setData(data, DataType.OCTET_STRING, _GXCommon.getBytes(it.response))
-                    _GXCommon.setData(data, DataType.UINT16, it.delay)
+                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, _GXCommon.getBytes(it.request))
+                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, _GXCommon.getBytes(it.response))
+                    _GXCommon.setData(settings, data, DataType.UINT16, it.delay)
             return data
         if e.index == 4:
             data = GXByteBuffer()
@@ -150,7 +150,7 @@ class GXDLMSModemConfiguration(GXDLMSObject, IGXDLMSBase):
             _GXCommon.setObjectCount(cnt, data)
             if cnt != 0:
                 for it in self.modemProfile:
-                    _GXCommon.setData(data, DataType.OCTET_STRING, _GXCommon.getBytes(it))
+                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, _GXCommon.getBytes(it))
             return data
         e.error = ErrorCode.READ_WRITE_DENIED
         return None
